@@ -1,9 +1,9 @@
 #       enparse - read RIPE database and check syntax errors
 #
 #	$RCSfile: enparse.pl,v $
-#	$Revision: 1.7 $
-#	$Author: david $
-#	$Date: 1995/02/10 11:39:22 $
+#	$Revision: 1.8 $
+#	$Author: ripe-dbm $
+#	$Date: 1995/04/18 13:58:03 $
 #
 #	ARGUMENTS:	filehandle to read from
 #	RETURNS:	INTEGER object_status
@@ -231,7 +231,7 @@ sub enparse {
     if (!($hasdelete = &hasdelete(*entry))) {
        print STDERR "enparse - checking object format\n" if $opt_V;
        $rtcode = &checkobject(*entry);
-       if ($rtcode == $O_OK) {
+       if ($rtcode != $O_ERROR) {
           print STDERR "enparse - checking object syntax\n" if $opt_V;
 	  $rtcode = &checksyntax(*entry);
        }
