@@ -3,11 +3,13 @@
 #       whoisd - whois Internet daemon
 #
 #	$RCSfile: whoisd.pl,v $
-#	$Revision: 0.48 $
-#	$Author: marten $
-#	$Date: 1995/02/03 11:17:46 $
+#	$Revision: 0.52 $
+#	$Author: dfk $
+#	$Date: 1995/02/26 09:23:15 $
 #
 
+$port = 43;
+
 @INC = ("LIBDIR", @INC);
 
 require "getopts.pl";
@@ -379,7 +381,7 @@ sub parse {
 
     $string =~ s/^\s+//;
 
-    if ($string =~ /^help/) {
+    if ($string =~ /^[hH][eE][lL][pP]\s*$/) {
 	open (HELP, $HELP);
 	while (<HELP>) {
 	    print;
@@ -400,6 +402,10 @@ sub parse {
 	    }
 	    next;
 	}
+	if ($string =~ s/^-U\s+(\S+)\s*//) {
+	    $opt_U = $1;
+	    next;
+	}
 	if ($string =~ s/^-(s)\s+(\S+)\s*//) {
 	    local($src) = $2;
 	    $src =~ tr/a-z/A-Z/;
@@ -465,6 +471,63 @@ sub parse {
 }
 
 
+sub whoisupdate {
+	local($name, $addr, $logstr) = @_;
+	local($ent) = "";
+
+	print STDERR "whoisupdate('$name', '$addr', '$logstr');\n" if $debug;
+	print STDERR "$BINDIR/dbupdate will be used.\n" if $debug;
+
+	if ($addr !~ /^193\.0\.0\.1[2345][0-9]/) {  # needs to go into config
+		print NS "$NOMATCH\n";
+		close(NS);
+		exit(1);
+	}
+		
+	while(<NS>) {
+		last if /^\.\s*$/;
+		if (/^\s*$/) {
+			if ($ent =~ /\S+/) {
+				&dowhoisupdate($name, $addr, $logstr, $ent);
+				$ent = "";
+			}
+		}
+		else {
+			$ent .= $_;
+		}
+	}
+}
+
+sub dowhoisupdate {
+	local($name, $addr, $logstr, $ent) = @_;
+
+	$logstr =~ s/[^A-Za-z0-9]//g;  # no shell spoofing !
+
+	print STDERR "dowhoisupdate('$name', '$addr', '$logstr');\n" if $debug;
+	print STDERR "dowhoisupdate \$ent:\n$ent\n" if $debug;
+
+
+	unlink("$TMPDIR/wupd.$$");
+	open(ENT, ">$TMPDIR/wupd.$$") || &nsexit("open $TMPDIR/wupd.$$: $!");
+	print ENT $ent;
+	close(ENT);
+
+	open(UPD, "$BINDIR/dbupdate -F -v -n 'NETWORK $addr $name $logstr' <$TMPDIR/wupd.$$|") || &nsexit("pipe $BINDIR/dbupdate: $!");
+	while(<UPD>) {
+		print NS $_;
+	}
+	close(UPD);
+	unlink("$TMPDIR/wupd.$$");
+}
+
+
+sub nsexit {
+	local($msg) = @_;
+
+	print NS "ERROR: $msg\n";
+	close(NS);
+	exit 1;
+}
 
 #
 # Main program
@@ -601,7 +664,13 @@ for (;;) {
 		select(NS);
 		$string = &parse($_);
 		print STDERR "($$) lookup $string\n" if $debug;
-		if (!$opt_t) {
+
+		if($opt_U) {
+		    alarm 60;
+		    &whoisupdate($name, $rhost, $opt_U);
+		    close(NS);
+		}
+		elsif (!$opt_t) {
 		    &whois(*source, $string);
 		    select(NS);
 		    print $NOMATCH,"\n" if $result == 0;
@@ -633,7 +702,7 @@ for (;;) {
 	    # Log this query
 
 	    $flags = "";
-	    for $fl ("d","a","s","k","r","F","t","S","T","M","m","L") {
+	    for $fl ("d","a","s","k","r","F","t","S","T","U","M","m","L") {
 		if (eval "\$opt_$fl;") {
 		    if ($flags) {
 			$flags .= ":";
